@@ -9,11 +9,11 @@ File::Next - File-finding iterator
 
 =head1 VERSION
 
-Version 1.12
+Version 1.14
 
 =cut
 
-our $VERSION = '1.12';
+our $VERSION = '1.14';
 
 =head1 SYNOPSIS
 
@@ -181,7 +181,7 @@ a collection of variables.
 =back
 
 The descend filter is NOT applied to any directory names specified
-in as I<@starting_points> in the constructor.  For example,
+as I<@starting_points> in the constructor.  For example,
 
     my $iter = File::Next::files( { descend_filter => sub{0} }, '/tmp' );
 
@@ -192,8 +192,9 @@ By default, the I<descend_filter> is C<sub {1}>, or "always descend".
 =head2 error_handler => \&error_handler
 
 If I<error_handler> is set, then any errors will be sent through
-it.  By default, this value is C<CORE::die>.  This function must
-NOT return.
+it.  If the error is OS-related (ex. file not found, not permissions), the
+native error code is passed as a second argument.  By default, this value is
+C<CORE::die>.  This function must NOT return.
 
 =head2 warning_handler => \&warning_handler
 
@@ -228,7 +229,7 @@ directory you're about to follow is actually a symlink.
 
 =head2 nul_separated => [ 0 | 1 ]
 
-Used on by the C<from_file> iterator.  Specifies that the files
+Used by the C<from_file> iterator.  Specifies that the files
 listed in the input file are separated by NUL characters, as from
 the C<find> command with the C<-print0> argument.
 
@@ -246,7 +247,7 @@ BEGIN {
     %files_defaults = (
         file_filter     => undef,
         descend_filter  => undef,
-        error_handler   => sub { CORE::die @_ },
+        error_handler   => sub { CORE::die $_[0] },
         warning_handler => sub { CORE::warn @_ },
         sort_files      => undef,
         follow_symlinks => 1,
@@ -260,12 +261,12 @@ sub files {
     die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
 
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
-    my $filter = $parms->{file_filter};
 
     return sub {
+        my $filter = $parms->{file_filter};
         while (@queue) {
-            my ($dirname,$file,$fullpath) = splice( @queue, 0, 3 ); ## no critic (ProhibitMagicNumbers)
-            if ( -f $fullpath || -p $fullpath || $fullpath =~ m{^/dev/fd} ) {
+            my ($dirname,$file,$fullpath) = splice( @queue, 0, 3 );
+            if ( -f $fullpath || -p _ || $fullpath =~ m{^/dev/fd} ) {
                 if ( $filter ) {
                     local $_ = $file;
                     local $File::Next::dir = $dirname;
@@ -274,7 +275,7 @@ sub files {
                 }
                 return wantarray ? ($dirname,$file,$fullpath) : $fullpath;
             }
-            elsif ( -d _ ) {
+            if ( -d _ ) {
                 unshift( @queue, _candidate_files( $parms, $fullpath ) );
             }
         } # while
@@ -291,7 +292,7 @@ sub dirs {
 
     return sub {
         while (@queue) {
-            my (undef,undef,$fullpath) = splice( @queue, 0, 3 ); ## no critic (ProhibitMagicNumbers)
+            my (undef,undef,$fullpath) = splice( @queue, 0, 3 );
             if ( -d $fullpath ) {
                 unshift( @queue, _candidate_files( $parms, $fullpath ) );
                 return $fullpath;
@@ -306,11 +307,11 @@ sub everything {
     die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
 
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
-    my $filter = $parms->{file_filter};
 
     return sub {
+        my $filter = $parms->{file_filter};
         while (@queue) {
-            my ($dirname,$file,$fullpath) = splice( @queue, 0, 3 ); ## no critic (ProhibitMagicNumbers)
+            my ($dirname,$file,$fullpath) = splice( @queue, 0, 3 );
             if ( -d $fullpath ) {
                 unshift( @queue, _candidate_files( $parms, $fullpath ) );
             }
@@ -332,7 +333,7 @@ sub from_file {
 
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
     my $err  = $parms->{error_handler};
-    my $warn = $parms->{error_handler};
+    my $warn = $parms->{warn_handler};
 
     my $filename = $queue[1];
 
@@ -347,13 +348,13 @@ sub from_file {
     }
     else {
         if ( !open( $fh, '<', $filename ) ) {
-            $err->( "Unable to open $filename: $!" );
+            $err->( "Unable to open $filename: $!", $! + 0 );
             return undef;
         }
     }
-    my $filter = $parms->{file_filter};
 
     return sub {
+        my $filter = $parms->{file_filter};
         local $/ = $parms->{nul_separated} ? "\x00" : $/;
         while ( my $fullpath = <$fh> ) {
             chomp $fullpath;
@@ -434,7 +435,7 @@ sub _setup {
 
     # Any leftover keys are bogus
     for my $badkey ( keys %passed_parms ) {
-        my $sub = (caller(1))[3]; ## no critic (ProhibitMagicNumbers)
+        my $sub = (caller(1))[3];
         $parms->{error_handler}->( "Invalid option passed to $sub(): $badkey" );
     }
 
@@ -472,7 +473,7 @@ sub _candidate_files {
 
     my $dh;
     if ( !opendir $dh, $dirname ) {
-        $parms->{error_handler}->( "$dirname: $!" );
+        $parms->{error_handler}->( "$dirname: $!", $! + 0 );
         return;
     }
 
@@ -484,13 +485,13 @@ sub _candidate_files {
     for my $file ( grep { !exists $skip_dirs{$_} } readdir $dh ) {
         my $has_stat;
 
-        # Only do directory checking if we have a descend_filter
         my $fullpath = File::Spec->catdir( $dirname, $file );
         if ( !$follow_symlinks ) {
             next if -l $fullpath;
             $has_stat = 1;
         }
 
+        # Only do directory checking if we have a descend_filter
         if ( $descend_filter ) {
             if ( $has_stat ? (-d _) : (-d $fullpath) ) {
                 local $File::Next::dir = $fullpath;
@@ -594,6 +595,9 @@ All file-finding in this module is adapted from Mark Jason Dominus'
 marvelous I<Higher Order Perl>, page 126.
 
 Thanks also for bug fixes and typo finding to
+Gerhard Poul,
+Brian Fraser,
+Todd Rinaldo,
 Bruce Woodward,
 Christopher J. Madsen,
 Bernhard Fisseni
@@ -601,7 +605,7 @@ and Rob Hoelz.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005-2012 Andy Lester.
+Copyright 2005-2016 Andy Lester.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the Artistic License version 2.0.
